@@ -2,12 +2,10 @@
 # -*- coding: utf-8 -*-
 # @Author : 陈坤泽
 # @Email  : 877362867@qq.com
-# @Data   : 2020/05/30 22:43
+# @Date   : 2020/05/30 22:43
 
 
 import html
-import subprocess
-import pandas as pd
 
 from pyxllib.debug._1_typelib import *
 
@@ -53,11 +51,11 @@ def viewfiles(procname, *files, **kwargs):
         basename, ext = os.path.splitext(kwargs['filename'])
 
     for i, t in enumerate(files):
-        if File(t).is_file() or is_url(t):
+        if File(t) or is_url(t):
             ls.append(str(t))
         else:
-            bn = basename or ''
-            ls.append(File(bn, ext, File.TEMP).write(t, if_exists=kwargs.get('if_exists', 'error')).fullpath)
+            bn = basename or ...
+            ls.append(File(bn, Dir.TEMP, suffix=ext).write(t, if_exists=kwargs.get('if_exists', 'error')).to_str())
 
     # 2 调用程序（并计算外部操作时间）
     tictoc = TicToc()
@@ -102,20 +100,21 @@ def chrome(arg_, **kwargs):
             + f'内存消耗：{sys.getsizeof(arg_)}（递归子类总大小：{getasizeof(arg_)}）Byte ===='
         t = '<p>' + html.escape(t) + '</p>'
         content = arg.to_html(**kwargs)
-        filename = File('', '.html', File.TEMP).write(t + content, etag=True, if_exists='ignore')
-        viewfiles('chrome.exe', str(filename))
+        f = File(..., Dir.TEMP, suffix='.html').write(t + content)
+        f = f.rename(get_etag(str(f)) + '.html', if_exists='delete')
+        viewfiles('chrome.exe', str(f))
     elif getattr(arg, 'render', None):  # pyecharts 等表格对象，可以用render生成html表格显示
         try:
             name = arg.options['title'][0]['text']
         except (LookupError, TypeError):
             name = Datetime().strftime('%H%M%S_%f')
-        filename = File(name, '.html', File.TEMP).fullpath
+        filename = File(name, Dir.TEMP, suffix='.html').to_str()
         arg.render(path=filename)
         viewfiles('chrome.exe', filename)
     else:
-        name = Datetime().strftime('%H%M%S_%f')
-        filename = File(name, '.txt', File.TEMP).write(arg).fullpath
-        viewfiles('chrome.exe', filename)
+        f = File(..., Dir.TEMP, suffix='.txt').write(arg)
+        f = f.rename(get_etag(str(f)) + f.suffix, if_exists='delete')
+        viewfiles('chrome.exe', str(f))
 
 
 def view_jsons_kv(fd, files='**/*.json', encoding=None, max_items=10, max_value_length=100):
@@ -130,13 +129,13 @@ def view_jsons_kv(fd, files='**/*.json', encoding=None, max_items=10, max_value_
     """
     kvc = KeyValuesCounter()
     d = Dir(fd)
-    for p in d.select(files).filepaths:
+    for p in d.select(files).subfiles():
         # print(p)
         data = p.read(encoding=encoding, mode='.json')
         kvc.add(data, max_value_length=max_value_length)
-    p = File(r'demo_keyvaluescounter.html', root=File.TEMP)
-    p.write(kvc.to_html_table(max_items=max_items), if_exists='replace')
-    chrome(p.fullpath)
+    p = File(r'demo_keyvaluescounter.html', Dir.TEMP)
+    p.write(kvc.to_html_table(max_items=max_items), if_exists='delete')
+    chrome(p.to_str())
 
 
 def check_repeat_filenames(dir, key='stem', link=True):
@@ -154,7 +153,7 @@ def check_repeat_filenames(dir, key='stem', link=True):
     # 1 智能解析dir参数
     if not isinstance(dir, Dir):
         dir = Dir(dir)
-    if not dir.files:
+    if not dir.subs:
         dir = dir.select('**/*', type_='file')
 
     # 2 辅助函数，智能解析key参数
@@ -169,7 +168,7 @@ def check_repeat_filenames(dir, key='stem', link=True):
     # 3 制作df表格数据
     columns = ['key', 'suffix', 'filename']
     li = []
-    for f in dir.files:
+    for f in dir.subs:
         p = File(f)
         li.append([extract_key(p), p.suffix.lower(), f])
     df = pd.DataFrame.from_records(li, columns=columns)
