@@ -2,12 +2,11 @@
 # -*- coding: utf-8 -*-
 # @Author : 陈坤泽
 # @Email  : 877362867@qq.com
-# @Data   : 2020/06/01
+# @Date   : 2020/06/01
 
 
 import enum
 import html
-import subprocess
 
 import pandas as pd
 from bs4 import BeautifulSoup
@@ -148,8 +147,8 @@ def showdir(c, *, to_html=None, printf=True):
     # 4 使用chrome.exe浏览或输出到控制台
     #   这里底层可以封装一个chrome函数来调用，但是这个chrome需要依赖太多功能，故这里暂时手动简单调用
     if to_html:
-        filename = File(object_name, suffix='.html', root=File.TEMP). \
-            write(ensure_gbk(res), if_exists='replace').fullpath
+        filename = File(object_name, Dir.TEMP, suffix='.html'). \
+            write(ensure_gbk(res), if_exists='delete').to_str()
         try:
             subprocess.run(['chrome.exe', filename])
         except FileNotFoundError:
