@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # @Author : 陈坤泽
 # @Email  : 877362867@qq.com
-# @Data   : 2020/06/02 15:48
+# @Date   : 2020/06/02 15:48
 
 """调试功能，通用底层功能
 
@@ -40,7 +40,7 @@ def render_echart(ob, name, show=False):
     # 如果没有设置页面标题，则默认采用文件名作为标题
     if not ob.page_title or ob.page_title == 'Awesome-pyecharts':
         ob.page_title = name
-    f = ob.render(File(f'{name}.html', root=File.TEMP).fullpath)
+    f = ob.render(str(File(f'{name}.html', Dir.TEMP)))
     if show: chrome(f)
     return f
 
